# http://flask.pocoo.org/snippets/62/
import base64
from os import urandom
from urllib.parse import urlparse, urljoin
from flask import request
from itsdangerous import URLSafeTimedSerializer, BadSignature
from projects import app
from projects import login_manager
from projects.models import User


def is_safe_url(target):
    ref_url = urlparse(request.host_url)
    test_url = urlparse(urljoin(request.host_url, target))
    return test_url.scheme in ('http', 'https') and \
        ref_url.netloc == test_url.netloc


# LoginManager
@login_manager.user_loader
def load_user(user_id):
    user = User.get(user_id)
    return user


def generate_confirmation_token(email):
    serializer = URLSafeTimedSerializer(app.config['SECRET_KEY'])
    return serializer.dumps(email, salt=app.config['ONETIME_TOKEN_SALT'])


# A token is valid for a day
def confirm_token(token, expiration=86400):
    serializer = URLSafeTimedSerializer(app.config['SECRET_KEY'])
    try:
        email = serializer.loads(token,
                                 salt=app.config['ONETIME_TOKEN_SALT'],
                                 max_age=expiration)
    except BadSignature:
        return False
    return email


def unique_name_encoding(name):
    """
    :param name: raw string name
    :return: the raw string name is padded with a 10 byte urandom string
    extension
    """
    assert type(name) == str
    return name + "_" + str(base64.b64encode(urandom(10)))


def unique_name_decode(name):
    """
    :param name: encoded name
    :return:
    """
    assert type(name) == str
    idx = name.rfind('_')
    return name[:idx]



from flask_wtf import FlaskForm
from flask_wtf.file import FileRequired, FileAllowed
from wtforms import StringField, TextAreaField, FileField, \
    SelectMultipleField, widgets, PasswordField
from wtforms.validators import DataRequired, Regexp, Email, EqualTo, Length
from projects.conf import config


class MultiCheckboxField(SelectMultipleField):
    widget = widgets.ListWidget(prefix_label=False)
    option_widget = widgets.CheckboxInput()


class ProjectFormManager():
    form_class = None

    def register_form_class(self, class_name, module=None):
        # Look in external module if provided, else try in the
        # projects module
        if module:
            module_ = __import__(module)
            class_ = getattr(module_, class_name)
        else:
            module_ = __import__("projects")
            class_ = getattr(module_, class_name)
        self.form_class = class_

    def get_form_class(self):
        if self.form_class is None:
            self.register_form_class(
                config.get('PROJECTS', 'form_class'),
                config.get('PROJECTS', 'form_module',
                           **{'fallback': None}))
        return self.form_class


class DefaultProjectForm(FlaskForm):
    name = StringField('Name', validators=[DataRequired()])
    description = TextAreaField('Description')
    image = FileField('Project Image', validators=[
        FileRequired(),
        FileAllowed(['jpg', 'png'], 'Only JPG and PNG images are allowed')
    ])
    tags = StringField('Tags', validators=[
        DataRequired(),
        Regexp(r'' + config.get('PROJECTS', 'tags_regex'),
               message=config.get('PROJECTS', 'tags_regex_msg'))])


class AuthRequestForm(FlaskForm):
    email = StringField('Email', validators=[
        DataRequired(),
        Email(message='Invalid email address format'),
        Regexp(r'' + config.get('PROJECTS', 'auth_regex_username'),
               message=config.get('PROJECTS', 'auth_regex_msg'))])


class PasswordResetRequestForm(FlaskForm):
    email = StringField('Email', validators=[
        DataRequired(),
        Email(message='Invalid email address format'),
        Regexp(r'' + config.get('PROJECTS', 'auth_regex_username'),
               message=config.get('PROJECTS', 'auth_regex_msg'))])


class PasswordResetForm(AuthRequestForm):
    password = PasswordField('Password', validators=[
        DataRequired(), Length(min=8),
        EqualTo('confirm', message='Passwords must match')])
    confirm = PasswordField('Repeat Password')


class LoginForm(FlaskForm):
    email = StringField('Email', validators=[
        DataRequired(),
        Email(message='Invalid email address format'),
        Regexp(r'' + config.get('PROJECTS', 'auth_regex_username'),
               message=config.get('PROJECTS', 'auth_regex_msg'))])
    password = PasswordField('Password', validators=[DataRequired()])



# Common configuration
import os
import configparser

# Load config.ini
res_path = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                        '..', 'res', 'config.ini')
config = configparser.SafeConfigParser()
config.read(res_path)

# common static folder path
config.get('BASE', 'static_folder')
config.get('BASE', 'upload_folder')
config.get('BASE', 'db_lock')



import shelve
import uuid
import fnmatch
import fcntl
import operator
from projects_base.base import config


class ShelveObject:
    def __init__(self):
        # Key Identifier
        self._id = str(uuid.uuid4())
        # Type identifier, used for retrieving the relevant instances
        self._type = str(type(self))

    def save(self):
        with open(config.get('BASE', 'db_lock'), 'wb') as lock:
            # Lock before db write operation, blocking for now
            fcntl.flock(lock.fileno(), fcntl.LOCK_EX)
            with shelve.open(config.get('BASE', 'db_path')) as db:
                db[self._id] = {key: self.__dict__[key] for key in
                                self.__dict__.keys()}
            # Unlock after operation
            fcntl.flock(lock.fileno(), fcntl.LOCK_UN)

        return self._id

    # Returns a new instance of the calling class with the same _shelve id.
    @classmethod
    def get(cls, instance_id):
        _object = None
        try:
            with shelve.open(config.get('BASE', 'db_path')) as db:
                _object = cls(**db[instance_id])
        except KeyError:
            pass
        return _object

    # Returns a list of instances of the calling class
    @classmethod
    def get_all(cls):
        ids = []
        with shelve.open(config.get('BASE', 'db_path')) as db:
            ids = [key for key in db.keys() if db[key]['_type'] == str(cls)]
        return [cls.get(instance_id) for instance_id in ids]

    @classmethod
    def remove(cls, shelve_id):
        with open(config.get('BASE', 'db_lock'), 'wb') as lock:
            # Lock before db write operation, blocking for now
            fcntl.flock(lock.fileno(), fcntl.LOCK_EX)
            with shelve.open(config.get('BASE', 'db_path')) as db:
                db.pop(shelve_id)
            # Unlock after operation
            fcntl.flock(lock.fileno(), fcntl.LOCK_UN)

    @classmethod
    def clear(cls):
        for instance in cls.get_all():
            cls.remove(instance._id)

    @classmethod
    def get_with_attr(cls, attr, value, collection=None):
        result = []
        if not collection:
            collection = cls.get_all()
        for obj in collection:
            if hasattr(obj, attr):
                obj_attr = getattr(obj, attr)
                if isinstance(obj_attr, type(value)) and \
                        obj_attr == value:
                    result.append(obj)
                else:
                    if isinstance(obj_attr, (list, set, tuple, dict)):
                        if value in obj_attr:
                            result.append(obj)
        return result

    @classmethod
    def get_with_search(cls, key, value, collection=None):
        result = []
        search = "*" + value + "*"
        if not collection:
            collection = cls.get_all()
        for obj in collection:
            if hasattr(obj, key):
                if len(fnmatch.filter([obj.__dict__[key].lower()],
                                      search.lower())) > 0:
                    result.append(obj)
        return result

    @classmethod
    def get_with_first(cls, key, value, collection=None):
        if not collection:
            collection = cls.get_all()
        for obj in collection:
            if hasattr(obj, key):
                if obj.__dict__[key] == value:
                    return obj
        return None

    @classmethod
    def get_top_with(cls, key, collection=None, num=10):
        if not collection:
            collection = cls.get_all()
        distinct_count = {}
        for obj in collection:
            if hasattr(obj, key):
                struct = obj.__dict__[key]
                distinct_count.update(count_distinct_in(struct,
                                                        distinct_count))
        top_num = dict(sorted(distinct_count.items(),
                              key=operator.itemgetter(1),
                              reverse=True)[:num])
        return top_num


def count_distinct_in(struct, current_count=None):
    if not current_count:
        current_count = {}
    if isinstance(struct, str):
        if struct not in current_count:
            current_count[struct] = 1
        else:
            current_count[struct] += 1
    if isinstance(struct, list):
        for i in struct:
            return count_distinct_in(i, current_count)
    if isinstance(struct, dict):
        for k, v in struct.items():
            return count_distinct_in(k, current_count)
    return current_count



import os
import datetime
from flask import render_template, request, flash, redirect, url_for, \
    jsonify
from flask_login import login_user, logout_user, login_required, current_user
from flask_mail import Message
from werkzeug.datastructures import CombinedMultiDict
from werkzeug.utils import secure_filename
from bcrypt import hashpw, gensalt
from projects import mail, project_manager, projects_blueprint, app
from projects.conf import config
from projects.models import Project, User
from projects.forms import AuthRequestForm, PasswordResetRequestForm, \
    LoginForm, PasswordResetForm, FileRequired
from projects.helpers import unique_name_encoding, unique_name_decode
from projects_base.base.forms import TagsSearchForm
from projects.helpers import generate_confirmation_token, confirm_token


# Routes
@projects_blueprint.route('/')
@projects_blueprint.route('/projects', methods=['GET'])
def projects():
    form = TagsSearchForm()
    entities = Project.get_all()
    tags = Project.get_top_with('tags', num=10)
    return render_template('projects/projects.html',
                           title=config.get('PROJECTS', 'title'),
                           grid_header="{} {}".format(
                               config.get('PROJECTS', 'title'), "Projects"),
                           tags=list(tags.keys()),
                           objects=entities, form=form)


@projects_blueprint.route('/my_projects', methods=['GET'])
@login_required
def my_projects():
    form = TagsSearchForm()
    entities = [project for project in Project.get_all()
                if project._id in current_user.projects]

    return render_template('projects/projects.html', objects=entities,
                           form=form)


@projects_blueprint.route('/show/<object_id>', methods=['GET'])
def show(object_id):
    form_class = project_manager.get_form_class()
    form = form_class()
    entity = Project.get(object_id)
    if entity is None:
        flash("That project doesn't exist", 'danger')
        return redirect(url_for('.projects'))

    owner = False
    if current_user.is_authenticated and object_id in current_user.projects:
        owner = True
        for attr in entity.__dict__:
            if attr != '_id' and attr != '_type':
                form[attr].data = entity.__dict__[attr]
            # Workaround for image upload required, set the label
            # to the currently used image and disable it as a required field
            if attr == "image":
                form[attr].label.text = "Stored image is: " \
                                        + unique_name_decode(
                    entity.__dict__[attr])
                form[attr].flags = None
    else:
        # Translate area keys to values
        if hasattr(entity, 'area'):
            entity.area = [area[1] for area in form.area.choices
                           if area[0] in entity.area]
    return render_template('projects/project.html', object=entity, owner=owner,
                           form=form)


@projects_blueprint.route('/create_project', methods=['GET', 'POST'])
@login_required
def create():
    form_class = project_manager.get_form_class()
    form = form_class(CombinedMultiDict((request.files, request.form)))

    if form.validate_on_submit():
        f = form.image.data
        # Make sure the saved image is filename is unique
        filename = secure_filename(unique_name_encoding(f.filename))
        f.save(os.path.join(config.get('PROJECTS', 'upload_folder'), filename))
        # Remove special fields
        if form.__contains__('csrf_token'):
            form._fields.pop('csrf_token')
        form._fields.pop('image')
        # Save new instance
        new_instance = {key: field.data for key, field in
                        form.__dict__.items()
                        if hasattr(field, 'data')}

        new_instance['image'] = filename
        entity = Project(**new_instance)
        entity_id = entity.save()
        # Update user with new instance
        current_user.projects.append(entity_id)
        current_user.save()
        url = url_for('projects.show', object_id=entity_id, _external=True)
        flash("Your submission has been received,"
              " your metadata can be found at: " + url, 'success')
        return redirect(url)
    return render_template('projects/create_project.html', form=form)


@projects_blueprint.route('/update/<object_id>', methods=['POST'])
@login_required
def update(object_id):
    entity = Project.get(object_id)
    if entity is None:
        flash("That entity dosen't exist", 'danger')
        return redirect(url_for('projects.projects'))

    if object_id not in current_user.projects:
        flash("Your trying to update an entity that's not yours", 'danger')
        return redirect(url_for('projects.projects'))

    form_class = project_manager.get_form_class()
    form = form_class(CombinedMultiDict((request.files, request.form)))
    # Strip image upload validation on upload (Optional)
    form.image.validators = [validator for validator in form.image.validators
                             if type(validator) is not FileRequired]

    if form.validate_on_submit():
        # Only save the image if a new was submitted, else keep the old name
        f = form.image.data
        if f.filename != '':
            filename = secure_filename(unique_name_encoding(f.filename))
            f.save(os.path.join(config.get('PROJECTS', 'upload_folder'),
                                filename))
            # Remove old
            os.remove(os.path.join(config.get('PROJECTS', 'upload_folder'),
                                   entity.image))
        else:
            filename = entity.image

        # Update every attribute except _id, _type, and image
        disabled_updates = ['_id', '_type', 'image']
        for attr in entity.__dict__:
            if attr not in disabled_updates:
                entity.__dict__[attr] = form[attr].data
        entity.__dict__['image'] = filename
        entity_id = entity.save()

        url = url_for('projects.show', object_id=entity_id, _external=True)
        flash("Update Success, your data can be found at: " + url,
              'success')
        return redirect(url)
    form.image.flags = None
    return render_template('projects/project.html', object=entity, form=form)


@projects_blueprint.route('/delete/<object_id>', methods=['POST'])
@login_required
def delete(object_id):
    entity = Project.get(object_id)
    if entity is None:
        flash("That entity dosen't exist", 'danger')
        return redirect(url_for('projects.projects'))
    if object_id in current_user.projects:
        Project.remove(object_id)
        current_user.projects.remove(object_id)
        os.remove(os.path.join(config.get('PROJECTS', 'upload_folder'),
                               entity.image))
        current_user.save()
        flash("Entity: " + entity.name + " has been deleted", 'success')
    else:
        flash("Your trying to delete an entity you don't own", 'danger')
    return redirect(url_for('projects.projects'))


# Sends approval emails to every app.config['ADMINS_EMAIL']
@projects_blueprint.route('/request_auth', methods=['POST'])
def request_auth():
    form = AuthRequestForm(request.form)
    if form.validate_on_submit():
        # Send confirmation token
        user = User.get_with_first('email', form.email.data)
        if user is None:
            token = generate_confirmation_token(email=form.email.data)
            confirm_url = url_for('projects.approve_auth',
                                  token=token, _external=True)
            html = render_template('projects/email/activate_user.html',
                                   email=form.email.data,
                                   confirm_url=confirm_url)
            msg = Message(subject=form.email.data
                          + " requests {} Projects access".format(
                              config.get('PROJECTS', 'title')),
                          html=html,
                          recipients=app.config['ADMINS_EMAIL'],
                          sender=app.config['MAIL_USERNAME'])
            try:
                mail.send(msg)
            except TimeoutError:
                return jsonify(data={
                    'danger': 'Timed out before request could be sent'
                              ' to an admin for approval'})
            return jsonify(data={
                'success': 'Request successfully submitted'
                           ', awaiting admin approval'})
        else:
            response = jsonify(
                data={'danger': 'That email has already been granted access'})
            response.status_code = 400
            return response
    response = jsonify(data={'danger': ', '.join(
        ["{} - {}".format(attr, msg) for attr, errors in form.errors.items()
         for msg in errors])})
    response.status_code = 400
    return response


@projects_blueprint.route('/request_password_reset', methods=['POST'])
def request_password_reset():
    form = PasswordResetRequestForm(request.form)
    if form.validate_on_submit():
        user = User.get_with_first('email', form.email.data)
        if user is None:
            response = jsonify(data={'danger': 'That user does not exist'})
            response.status_code = 400
            return response
        else:
            email = user.email
            token = generate_confirmation_token(email=email)
            reset_url = url_for('projects.reset_password',
                                token=token, _external=True)
            html = render_template('projects/email/reset_password.html',
                                   email=email,
                                   reset_password_url=reset_url)
            msg = Message(subject='{} Reset Password'.format(
                config.get('PROJECTS', 'title')),
                html=html,
                recipients=[email],
                sender=app.config['MAIL_USERNAME'])
            mail.send(msg)
            return jsonify(
                data={'success': 'A password reset link has been sent to {}'
                      .format(email)})
    response = jsonify(data={'danger': ', '.join(
        ["{} - {}".format(attr, msg) for attr, errors in form.errors.items()
         for msg in errors])})
    response.status_code = 400
    return response


# Accepts approval from admin's
@projects_blueprint.route('/approve_auth/<token>')
def approve_auth(token):
    email = confirm_token(token)
    if email is False:
        flash('Confirmation failed, either it is invalid or expired.',
              'danger')
    else:
        user = User.get_with_first('email', email)
        if user is not None:
            flash('That email has already been registered')
        else:
            # Setup
            user = User(email=email,
                        password=hashpw(os.urandom(24), gensalt()),
                        projects=[], is_active=False, is_authenticated=False,
                        is_anonymous=False,
                        confirmed_on=datetime.datetime.now())
            user.save()

            token = generate_confirmation_token(email=email)
            reset_url = url_for('projects.reset_password',
                                token=token, _external=True)
            html = render_template('projects/email/reset_password.html',
                                   email=email,
                                   reset_password_url=reset_url)
            msg = Message(subject='{} Projects Account approval'.format(
                config.get('PROJECTS', 'title')),
                html=html,
                recipients=[email],
                sender=app.config['MAIL_USERNAME'])
            mail.send(msg)
            flash("The account {} has been approved and created".format(email),
                  'success')
    return redirect(url_for('projects.projects'))


@projects_blueprint.route('/reset_password/<token>', methods=['POST', 'GET'])
def reset_password(token):
    email = confirm_token(token)
    if email is False:
        flash(
            'Attempted password reset failed,'
            ' the request is either invalid or expired', 'danger')
    else:
        form = PasswordResetForm(request.form)
        form.email.data = email
        if form.validate_on_submit():
            user = User.get_with_first('email', email)
            user.is_active = True
            user.is_authenticated = True
            user.is_anonymous = False
            user.email = email
            user.password = hashpw(bytes(form.password.data, 'utf-8'),
                                   gensalt())
            user.save()
            flash('Your password has now been updated', 'success')
            return redirect(url_for('projects.projects'))
        return render_template('projects/reset_password_form.html', form=form)
    return redirect(url_for('projects.login'))


# @app.route('/login', methods=['GET', 'POST'])
@projects_blueprint.route('/login', methods=['GET', 'POST'])
def login():
    form = LoginForm(request.form)
    if form.validate_on_submit():
        valid_user = User.valid_user(form.email.data, form.password.data)
        if valid_user is not None:
            flash('Logged in successfully.', 'success')
            login_user(valid_user)
            return redirect(url_for('projects.projects'))
        else:
            flash('Invalid Credentials', 'danger')
    return render_template('projects/login.html', form=form)


@projects_blueprint.route('/logout')
@login_required
def logout():
    logout_user()
    return redirect(url_for('projects.projects'))


@projects_blueprint.route('/tag/<tag>', methods=['GET'])
def tag_search(tag):
    form = TagsSearchForm(data={'tag': tag}, csrf_enabled=False)
    entities = {}
    tags = Project.get_top_with('tags')
    if form.validate():
        entities = Project.get_with_search('tags', form.tag.data)
    return render_template('projects/projects.html', tags=list(tags.keys()),
                           objects=entities,
                           form=form)


# TODO -> refactor with fair search forms in common views instead.
@projects_blueprint.route('/search', methods=['GET'])
def tag_external_search():
    form = TagsSearchForm(request.args, csrf_enabled=False)
    entities = {}
    # The return form should contain a csrf_token
    return_form = TagsSearchForm()
    return_form.tag = form.tag
    if form.validate():
        entities = Project.get_with_search('tags', form.tag.data)
        return render_template('projects/projects.html', objects=entities,
                               form=return_form)
    # pass on errors
    return_form._errors = form.errors
    return_form._fields['tag'] = form._fields['tag']
    return render_template('projects/projects.html', objects=entities,
                           form=return_form)


# TODO -> refactor with fair search forms in common views instead.
@projects_blueprint.route('/search', methods=['POST'])
def tag_native_search():
    form = TagsSearchForm(request.form)
    if form.validate_on_submit():
        result = {}
        tag = request.form['tag']
        entities = Project.get_with_search('tags', tag)
        if len(entities) > 0:
            result = [entity.serialize() for entity in entities]
        return jsonify(data=result)

    response = jsonify(data={'danger': ', '.join(
        [msg for attr, errors in form.errors.items() for msg in errors])})
    response.status_code = 400
    return response



from projects_base.base.models import ShelveObject
from bcrypt import checkpw


class Project(ShelveObject):

    def __init__(self, **kwargs):
        super().__init__()
        for key, value in kwargs.items():
            self.__dict__[key] = value

    def serialize(self):
        return self.__dict__


class User(ShelveObject):

    def __init__(self, **kwargs):
        super().__init__()
        for key, value in kwargs.items():
            self.__dict__[key] = value

    def serialize(self):
        return self.__dict__

    @staticmethod
    def valid_user(email, password):
        user = User.get_with_first('email', email)
        if user is not None and checkpw(bytes(password, 'utf-8'),
                                        user.password):
            return user
        return None

    def is_authenticated(self):
        return self.__dict__['is_authenticated']

    def is_active(self):
        return self.__dict__['is_active']

    def is_anonymous(self):
        return self.__dict__['is_anonymous']

    def get_id(self):
        return self._id



from flask_wtf import FlaskForm
from wtforms import StringField
from wtforms.validators import Optional, Regexp


class TagsSearchForm(FlaskForm):
    tag = StringField('Tags Search',
                      validators=[Optional(),
                                  Regexp(r'^[\w,:_\-* ]*$',
                                         message="Allowed tag characters "
                                                 "include letters spaces and"
                                                 ", : _ - *")])



from flask_login import current_user
from flask_nav.elements import Navbar, View
from projects.conf import config
from projects import nav


@nav.navigation()
def nav_bar():
    navbar = list(Navbar(
        View('{}'.format(config.get('PROJECTS', 'title')),
             '.projects'),
        View('Projects', '.projects'),
    ).items)
    if current_user.is_authenticated:
        navbar.extend([
            View('My Projects', 'projects.my_projects'),
            View('Create Project', 'projects.create'),
            View('Logout', 'projects.logout'),
        ])
    else:
        navbar.extend([
            View('Login', 'projects.login'),
        ])

    return Navbar('{}'.format(config.get('PROJECTS', 'title')),
                  *navbar)



import os
from flask import Flask, Blueprint
from flask_wtf.csrf import CSRFProtect
from flask_bootstrap import Bootstrap
from flask_login import LoginManager
from flask_mail import Mail
from flask_nav import Nav
from projects_base.base import base_blueprint
from projects.models import User
from projects.forms import ProjectFormManager, DefaultProjectForm
from projects.conf import config

app = Flask(__name__)
app.register_blueprint(base_blueprint)
projects_blueprint = Blueprint('projects', __name__,
                               static_folder='static',
                               static_url_path='/projects/static',
                               template_folder='templates')

csrf = CSRFProtect(app)
app.secret_key = os.urandom(24)
Bootstrap(app)

# Login Manager
login_manager = LoginManager()
login_manager.init_app(app)
login_manager.login_view = 'login'

project_manager = ProjectFormManager()

# Onetime authentication reset token salt
app.config['ONETIME_TOKEN_SALT'] = os.urandom(24)

# TODO -> fix hack at some point
# Application ADMINS_EMAIL
if 'ADMINS_EMAIL' in os.environ:
    app.config['ADMINS_EMAIL'] = os.environ['ADMINS_EMAIL'].split(',')
else:
    app.config['ADMINS_EMAIL'] = config.get('MAIL', 'admins')

# Email application server
if 'MAIL_SERVER' in os.environ:
    app.config['MAIL_SERVER'] = os.environ['MAIL_SERVER']
else:
    app.config['MAIL_SERVER'] = config.get('MAIL', 'server')

if 'MAIL_PORT' in os.environ:
    app.config['MAIL_PORT'] = os.environ['MAIL_PORT']
else:
    app.config['MAIL_PORT'] = config.get('MAIL', 'port')

if 'MAIL_USE_TLS' in os.environ:
    app.config['MAIL_USE_TLS'] = bool(os.environ['MAIL_USE_TLS'])
else:
    app.config['MAIL_USE_TLS'] = bool(config.get('MAIL', 'use_tls'))

if 'MAIL_USE_SSL' in os.environ:
    app.config['MAIL_USE_SSL'] = False \
        if os.environ['MAIL_USE_SSL'] == 'False' else True
else:
    app.config['MAIL_USE_SSL'] = bool(config.get('MAIL', 'use_ssl'))

if 'MAIL_USERNAME' in os.environ:
    app.config['MAIL_USERNAME'] = os.environ['MAIL_USERNAME']
else:
    app.config['MAIL_USERNAME'] = config.get('MAIL', 'username')

if 'MAIL_PASSWORD' in os.environ:
    app.config['MAIL_PASSWORD'] = os.environ['MAIL_PASSWORD']
else:
    app.config['MAIL_PASSWORD'] = config.get('MAIL', 'password')

# Connect mail
mail = Mail(app)

# Setup navbar
nav = Nav()
nav.init_app(app)
import projects.nav
import projects.views
app.register_blueprint(projects_blueprint)



from flask import Blueprint
from projects_base.base.conf import config

base_blueprint = Blueprint('projects_base', __name__,
                           static_folder='static',
                           static_url_path='/base/static',
                           template_folder='templates')
import projects_base.base.views



import os
from projects_base.base.conf import config

config.read(os.path.abspath("projects_base/res/config.ini"))
config.read(os.path.abspath("res/config.ini"))

# Required folders
folders = {}
folders['DATA_FOLDER'] = os.path.abspath(config.get('PROJECTS',
                                                    'data_folder'))
folders['UPLOAD_FOLDER'] = os.path.abspath(config.get('PROJECTS',
                                                      'upload_folder'))

# Create required folders for the application if they don't exist
for key, folder in folders.items():
    try:
        os.makedirs(folder)
        print("Created: " + folder)
    except FileExistsError:
        pass



import os
from projects_base.base import base_blueprint
from projects_base.base.conf import config
from flask import send_from_directory


@base_blueprint.route('/images/<path:filename>', methods=['GET'])
def img_render(filename):
    return send_from_directory(os.path.abspath(
        config.get('BASE', 'upload_folder')), filename=filename)



# http://flask.pocoo.org/docs/0.12/patterns/distribute/
import os
from setuptools import find_packages
from distutils.core import setup

here = os.path.dirname(__file__)
long_description = open('README.rst').read()

version_ns = {}
with open(os.path.join(here, 'version.py')) as f:
    exec(f.read(), {}, version_ns)

setup(
    name='projects-site',
    version=version_ns['__version__'],
    long_description=long_description,
    description="""
                A website template for hosting project
                metadata information via flask
                """,
    url='https://github.com/rasmunk/projects-site',
    author='Rasmus Munk',
    author_email='rasmus.munk@nbi.ku.dk',
    classifiers=[
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ],
    keywords=['Website', 'Flask', 'MetaData'],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Flask',
        'flask-bootstrap',
        'flask-nav',
        'flask_wtf',
        'wtforms',
        'requests',
        'wtforms-components',
        'flask_login',
        'flask_mail',
        'bcrypt',
        'configparser'
    ],
    setup_requires=[
        'pytest-runner',
    ],
    tests_require=[
        'pytest',
    ],
)
